@@ -1,5 +1,5 @@
-#ifndef __SETUP_H
-#define __SETUP_H
+#ifndef __LIB_CURL_SETUP_H
+#define __LIB_CURL_SETUP_H
 /***************************************************************************
  *                                  _   _ ____  _
  *  Project                     ___| | | |  _ \| |
@@ -7,7 +7,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2005, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2006, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -20,49 +20,88 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: setup.h,v 1.91 2005/10/02 16:52:07 giva Exp $
+ * $Id: setup.h,v 1.110 2006-06-08 06:12:31 bagder Exp $
  ***************************************************************************/
 
 #ifdef HTTP_ONLY
+#define CURL_DISABLE_TFTP
 #define CURL_DISABLE_FTP
 #define CURL_DISABLE_LDAP
 #define CURL_DISABLE_TELNET
 #define CURL_DISABLE_DICT
 #define CURL_DISABLE_FILE
-#define CURL_DISABLE_GOPHER
 #endif /* HTTP_ONLY */
 
 #if !defined(WIN32) && defined(__WIN32__)
-/* This should be a good Borland fix. */
-#define WIN32
-#endif
-#if !defined(WIN32) && defined(_WIN32)
-/* This works for VS2005 on x64 */
+/* Borland fix */
 #define WIN32
 #endif
 
+#if !defined(WIN32) && defined(_WIN32)
+/* VS2005 on x64 fix */
+#define WIN32
+#endif
+
+/*
+ * Include configuration script results or hand-crafted
+ * configuration file for platforms which lack config tool.
+ */
+
 #ifdef HAVE_CONFIG_H
-#include "config.h" /* the configure script results */
+#include "config.h"
 #else
+
 #ifdef _WIN32_WCE
 #include "config-win32ce.h"
 #else
 #ifdef WIN32
-/* hand-modified win32 config.h! */
 #include "config-win32.h"
-#endif /* WIN32 */
-#endif /* WIN32_WCE */
-#endif /* HAVE_CONFIG_H */
+#endif
+#endif
 
 #ifdef macintosh
-/* hand-modified MacOS config.h! */
 #include "config-mac.h"
 #endif
+
 #ifdef AMIGA
-/* hand-modified AmigaOS config.h! */
 #include "amigaos.h"
 #endif
 
+#ifdef TPF
+#include "config-tpf.h" /* hand-modified TPF config.h */
+/* change which select is used for libcurl */
+#define select(a,b,c,d,e) tpf_select_libcurl(a,b,c,d,e)
+#endif
+
+#endif /* HAVE_CONFIG_H */
+
+/*
+ * Include header files for windows builds before redefining anything.
+ * Use this preproessor block only to include or exclude windows.h,
+ * winsock2.h, ws2tcpip.h or winsock.h. Any other windows thing belongs
+ * to any other further and independant block.  Under Cygwin things work
+ * just as under linux (e.g. <sys/socket.h>) and the winsock headers should
+ * never be included.
+ */
+
+#if defined(HAVE_WINDOWS_H) && !defined(__CYGWIN__)
+#  ifndef WIN32_LEAN_AND_MEAN
+#    define WIN32_LEAN_AND_MEAN
+#  endif
+#  include <windows.h>
+#  ifdef HAVE_WINSOCK2_H
+#    include <winsock2.h>
+#    ifdef HAVE_WS2TCPIP_H
+#       include <ws2tcpip.h>
+#    endif
+#  else
+#    ifdef HAVE_WINSOCK_H
+#      include <winsock.h>
+#    endif
+#  endif
+#endif
+
+
 #ifndef TRUE
 #define TRUE 1
 #endif
@@ -121,6 +160,32 @@ typedef unsigned char bool;
 #include <curl/stdcheaders.h>
 #endif
 
+/*
+ * PellesC cludge section (yikes);
+ *  - It has 'ssize_t', but it is in <unistd.h>. The way the headers
+ *    on Win32 are included, forces me to include this header here.
+ *  - sys_nerr, EINTR is missing in v4.0 or older.
+ */
+#ifdef __POCC__
+  #include <sys/types.h>
+  #include <unistd.h>
+  #if (__POCC__ <= 400)
+  #define sys_nerr EILSEQ  /* for strerror.c */
+  #define EINTR    -1      /* for select.c */
+  #endif
+#endif
+
+/*
+ * Salford-C cludge section (mostly borrowed from wxWidgets).
+ */
+#ifdef __SALFORDC__
+  #pragma suppress 353             /* Possible nested comments */
+  #pragma suppress 593             /* Define not used */
+  #pragma suppress 61              /* enum has no name */
+  #pragma suppress 106             /* unnamed, unused parameter */
+  #include <clib.h>
+#endif
+
 #if defined(CURLDEBUG) && defined(HAVE_ASSERT_H)
 #define curlassert(x) assert(x)
 #else
@@ -149,6 +214,7 @@ typedef unsigned char bool;
 #define struct_stat struct stat
 #endif /* Win32 with large file support */
 
+
 /* Below we define four functions. They should
    1. close a socket
    2. read from a socket
@@ -160,18 +226,6 @@ typedef unsigned char bool;
 
 #ifdef WIN32
 
-#ifndef WIN32_LEAN_AND_MEAN
-#define WIN32_LEAN_AND_MEAN  /* Prevent including <winsock*.h> in <windows.h> */
-#endif /* WIN32_LEAN_AND_MEAN */
-
-#ifdef HAVE_WINSOCK2_H
-#include <winsock2.h>        /* required by telnet.c */
-#endif
-
-#if defined(ENABLE_IPV6) || defined(USE_SSLEAY)
-#include <ws2tcpip.h>
-#endif
-
 #if !defined(__GNUC__) || defined(__MINGW32__)
 #define sclose(x) closesocket(x)
 
@@ -194,10 +248,12 @@ typedef unsigned char bool;
 #else /* WIN32 */
 
 #ifdef DJGPP
+#include <sys/ioctl.h>
 #define sclose(x)         close_s(x)
 #define sread(x,y,z)      read_s(x,y,z)
 #define swrite(x,y,z)     write_s(x,y,z)
 #define select(n,r,w,x,t) select_s(n,r,w,x,t)
+#define ioctl(x,y,z) ioctlsocket(x,y,(char *)(z))
 #define IOCTL_3_ARGS
 #include <tcp.h>
 #ifdef word
@@ -227,7 +283,9 @@ typedef unsigned char bool;
 #endif
 
 #define DIR_CHAR      "/"
+#ifndef DOT_CHAR
 #define DOT_CHAR      "."
+#endif
 
 #ifdef DJGPP
 #undef DOT_CHAR
@@ -240,6 +298,7 @@ int fileno( FILE *stream);
 
 #endif /* WIN32 */
 
+#ifndef curl_socket_typedef
 /* now typedef our socket type */
 #ifdef WIN32
 typedef SOCKET curl_socket_t;
@@ -248,6 +307,9 @@ typedef SOCKET curl_socket_t;
 typedef int curl_socket_t;
 #define CURL_SOCKET_BAD -1
 #endif
+#define curl_socket_typedef
+#endif /* curl_socket_typedef */
+
 
 #if defined(ENABLE_IPV6) && defined(USE_ARES)
 #error "ares does not yet support IPv6. Disable IPv6 or ares and rebuild"
@@ -264,12 +326,24 @@ typedef int curl_socket_t;
 
 /* "cl -ML" or "cl -MLd" implies a single-threaded runtime library where
    _beginthreadex() is not available */
-#if defined(_MSC_VER) && !defined(_MT) && !defined(USE_ARES)
+#if (defined(_MSC_VER) && !defined(__POCC__)) && !defined(_MT) && !defined(USE_ARES)
 #undef USE_THREADING_GETADDRINFO
 #undef USE_THREADING_GETHOSTBYNAME
 #define CURL_NO__BEGINTHREADEX
 #endif
 
+/*
+ * msvc 6.0 does not have struct sockaddr_storage and
+ * does not define IPPROTO_ESP in winsock2.h. But both
+ * are available if PSDK is properly installed.
+ */
+
+#ifdef _MSC_VER
+#if !defined(HAVE_WINSOCK2_H) || ((_MSC_VER < 1300) && !defined(IPPROTO_ESP))
+#undef HAVE_STRUCT_SOCKADDR_STORAGE
+#endif
+#endif
+
 #ifdef mpeix
 #define IOCTL_3_ARGS
 #endif
@@ -299,9 +373,11 @@ typedef int curl_socket_t;
 #define USE_SSL    /* Either OpenSSL || GnuTLS */
 #endif
 
+#if !defined(CURL_DISABLE_HTTP) && !defined(CURL_DISABLE_NTLM)
 #if defined(USE_SSLEAY) || defined(USE_WINDOWS_SSPI)
 #define USE_NTLM
 #endif
+#endif
 
 #ifdef CURLDEBUG
 #define DEBUGF(x) x
@@ -309,4 +385,4 @@ typedef int curl_socket_t;
 #define DEBUGF(x)
 #endif
 
-#endif /* __CONFIG_H */
+#endif /* __LIB_CURL_SETUP_H */
