@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: tftp.c,v 1.17 2006-03-20 07:32:50 bagder Exp $
+ * $Id: tftp.c,v 1.26 2006-05-09 13:02:53 bagder Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -36,7 +36,6 @@
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
-#include <errno.h>
 
 #if defined(WIN32)
 #include <time.h>
@@ -86,6 +85,9 @@
 /* The last #include file should be: */
 #include "memdebug.h"
 
+/* RFC2348 allows the block size to be negotiated, but we don't support that */
+#define TFTP_BLOCKSIZE 512
+
 typedef enum {
   TFTP_MODE_NETASCII=0,
   TFTP_MODE_OCTET
@@ -122,23 +124,7 @@ typedef enum {
 } tftp_error_t;
 
 typedef struct tftp_packet {
-  unsigned short  event;
-  union {
-    struct {
-      unsigned char   data[512];
-    } request;
-    struct {
-      unsigned short  block;
-      unsigned char   data[512];
-    } data;
-    struct {
-      unsigned short  block;
-    } ack;
-    struct {
-      unsigned short  code;
-      unsigned char   data[512];
-    } error;
-  } u;
+  unsigned char data[2 + 2 + TFTP_BLOCKSIZE];
 } tftp_packet_t;
 
 typedef struct tftp_state_data {
@@ -165,8 +151,8 @@ typedef struct tftp_state_data {
 
 
 /* Forward declarations */
-static void tftp_rx(tftp_state_data_t *state, tftp_event_t event) ;
-static void tftp_tx(tftp_state_data_t *state, tftp_event_t event) ;
+static CURLcode tftp_rx(tftp_state_data_t *state, tftp_event_t event) ;
+static CURLcode tftp_tx(tftp_state_data_t *state, tftp_event_t event) ;
 void tftp_set_timeouts(tftp_state_data_t *state) ;
 
 /**********************************************************
@@ -199,7 +185,8 @@ void tftp_set_timeouts(tftp_state_data_t *state)
 
     /* Compute the re-start interval to suit the timeout */
     state->retry_time = timeout/state->retry_max;
-    if(state->retry_time<1) state->retry_time=1;
+    if(state->retry_time<1)
+      state->retry_time=1;
 
   }
   else {
@@ -215,12 +202,16 @@ void tftp_set_timeouts(tftp_state_data_t *state)
     state->retry_max = timeout/15;
   }
   /* But bound the total number  */
-  if(state->retry_max<3) state->retry_max=3;
-  if(state->retry_max>50) state->retry_max=50;
+  if(state->retry_max<3)
+    state->retry_max=3;
+
+  if(state->retry_max>50)
+    state->retry_max=50;
 
   /* Compute the re-ACK interval to suit the timeout */
   state->retry_time = timeout/state->retry_max;
-  if(state->retry_time<1) state->retry_time=1;
+  if(state->retry_time<1)
+    state->retry_time=1;
 
   infof(data, "set timeouts for state %d; Total %d, retry %d maxtry %d\n",
         state->state, (state->max_time-state->start_time),
@@ -235,12 +226,39 @@ void tftp_set_timeouts(tftp_state_data_t *state)
  *
  **********************************************************/
 
-static void tftp_send_first(tftp_state_data_t *state, tftp_event_t event)
+static void setpacketevent(tftp_packet_t *packet, unsigned short num)
+{
+  packet->data[0] = (num >> 8);
+  packet->data[1] = (num & 0xff);
+}
+
+
+static void setpacketblock(tftp_packet_t *packet, unsigned short num)
+{
+  packet->data[2] = (num >> 8);
+  packet->data[3] = (num & 0xff);
+}
+
+static unsigned short getrpacketevent(tftp_packet_t *packet)
+{
+  return (packet->data[0] << 8) | packet->data[1];
+}
+
+static unsigned short getrpacketblock(tftp_packet_t *packet)
+{
+  return (packet->data[2] << 8) | packet->data[3];
+}
+
+static CURLcode tftp_send_first(tftp_state_data_t *state, tftp_event_t event)
 {
   int sbytes;
   const char *mode = "octet";
-  char *filename = state->conn->path;
+
+  /* As RFC3617 describes the separator slash is not actually part of the file
+     name so we skip the always-present first letter of the path string. */
+  char *filename = &state->conn->path[1];
   struct SessionHandle *data = state->conn->data;
+  CURLcode res = CURLE_OK;
 
   /* Set ascii mode if -B flag was used */
   if(data->set.ftp_ascii)
@@ -255,24 +273,23 @@ static void tftp_send_first(tftp_state_data_t *state, tftp_event_t event)
     if(state->retries>state->retry_max) {
       state->error = TFTP_ERR_NORESPONSE;
       state->state = TFTP_STATE_FIN;
-      return;
+      return res;
     }
 
     if(data->set.upload) {
       /* If we are uploading, send an WRQ */
-      state->spacket.event = htons(TFTP_EVENT_WRQ);
-      filename = curl_unescape(filename, (int)strlen(filename));
-      state->conn->upload_fromhere = (char *)state->spacket.u.data.data;
-      if(data->set.infilesize != -1) {
+      setpacketevent(&state->spacket, TFTP_EVENT_WRQ);
+      filename = curl_easy_unescape(data, filename, 0, NULL);
+      state->conn->upload_fromhere = (char *)&state->spacket.data[4];
+      if(data->set.infilesize != -1)
         Curl_pgrsSetUploadSize(data, data->set.infilesize);
-      }
     }
     else {
       /* If we are downloading, send an RRQ */
-      state->spacket.event = htons(TFTP_EVENT_RRQ);
+      setpacketevent(&state->spacket, TFTP_EVENT_RRQ);
     }
-    snprintf((char *)state->spacket.u.request.data,
-             sizeof(state->spacket.u.request.data),
+    snprintf((char *)&state->spacket.data[2],
+             TFTP_BLOCKSIZE,
              "%s%c%s%c", filename, '\0',  mode, '\0');
     sbytes = 4 + (int)strlen(filename) + (int)strlen(mode);
     sbytes = sendto(state->sockfd, (void *)&state->spacket,
@@ -280,7 +297,7 @@ static void tftp_send_first(tftp_state_data_t *state, tftp_event_t event)
                     state->conn->ip_addr->ai_addr,
                     state->conn->ip_addr->ai_addrlen);
     if(sbytes < 0) {
-      failf(data, "%s\n", strerror(errno));
+      failf(data, "%s\n", Curl_strerror(state->conn, Curl_sockerrno()));
     }
     break;
 
@@ -288,15 +305,13 @@ static void tftp_send_first(tftp_state_data_t *state, tftp_event_t event)
     infof(data, "%s\n", "Connected for transmit");
     state->state = TFTP_STATE_TX;
     tftp_set_timeouts(state);
-    tftp_tx(state, event);
-    break;
+    return tftp_tx(state, event);
 
   case TFTP_EVENT_DATA: /* connected for receive */
     infof(data, "%s\n", "Connected for receive");
     state->state = TFTP_STATE_RX;
     tftp_set_timeouts(state);
-    tftp_rx(state, event);
-    break;
+    return tftp_rx(state, event);
 
   case TFTP_EVENT_ERROR:
     state->state = TFTP_STATE_FIN;
@@ -306,6 +321,7 @@ static void tftp_send_first(tftp_state_data_t *state, tftp_event_t event)
     failf(state->conn->data, "tftp_send_first: internal error\n");
     break;
   }
+  return res;
 }
 
 /**********************************************************
@@ -315,7 +331,7 @@ static void tftp_send_first(tftp_state_data_t *state, tftp_event_t event)
  * Event handler for the RX state
  *
  **********************************************************/
-static void tftp_rx(tftp_state_data_t *state, tftp_event_t event)
+static CURLcode tftp_rx(tftp_state_data_t *state, tftp_event_t event)
 {
   int sbytes;
   int rblock;
@@ -326,7 +342,7 @@ static void tftp_rx(tftp_state_data_t *state, tftp_event_t event)
   case TFTP_EVENT_DATA:
 
     /* Is this the block we expect? */
-    rblock = ntohs(state->rpacket.u.data.block);
+    rblock = getrpacketblock(&state->rpacket);
     if ((state->block+1) != rblock) {
       /* No, log it, up the retry count and fail if over the limit */
       infof(data,
@@ -335,23 +351,23 @@ static void tftp_rx(tftp_state_data_t *state, tftp_event_t event)
       if (state->retries>state->retry_max) {
         failf(data, "tftp_rx: giving up waiting for block %d\n",
               state->block+1);
-        return;
+        return CURLE_TFTP_ILLEGAL;
       }
     }
     /* This is the expected block.  Reset counters and ACK it. */
     state->block = rblock;
     state->retries = 0;
-    state->spacket.event = htons(TFTP_EVENT_ACK);
-    state->spacket.u.ack.block = htons(state->block);
-    sbytes = sendto(state->sockfd, (void *)&state->spacket,
+    setpacketevent(&state->spacket, TFTP_EVENT_ACK);
+    setpacketblock(&state->spacket, state->block);
+    sbytes = sendto(state->sockfd, (void *)state->spacket.data,
                     4, SEND_4TH_ARG,
                     (struct sockaddr *)&state->remote_addr,
                     state->remote_addrlen);
     if(sbytes < 0) {
-      failf(data, "%s\n", strerror(errno));
+      failf(data, "%s\n", Curl_strerror(state->conn, Curl_sockerrno()));
     }
 
-    /* Check if completed (That is, a less than full packet is recieved) */
+    /* Check if completed (That is, a less than full packet is received) */
     if (state->rbytes < (int)sizeof(state->spacket)){
       state->state = TFTP_STATE_FIN;
     }
@@ -368,7 +384,8 @@ static void tftp_rx(tftp_state_data_t *state, tftp_event_t event)
     if(state->retries > state->retry_max) {
       state->error = TFTP_ERR_TIMEOUT;
       state->state = TFTP_STATE_FIN;
-    } else {
+    }
+    else {
       /* Resend the previous ACK */
       sbytes = sendto(state->sockfd, (void *)&state->spacket,
                       4, SEND_4TH_ARG,
@@ -376,7 +393,7 @@ static void tftp_rx(tftp_state_data_t *state, tftp_event_t event)
                       state->remote_addrlen);
       /* Check all sbytes were sent */
       if(sbytes<0) {
-        failf(data, "%s\n", strerror(errno));
+        failf(data, "%s\n", Curl_strerror(state->conn, Curl_sockerrno()));
       }
     }
     break;
@@ -390,7 +407,8 @@ static void tftp_rx(tftp_state_data_t *state, tftp_event_t event)
     break;
   }
   Curl_pgrsSetDownloadCounter(data,
-                              (curl_off_t) state->block*512);
+                              (curl_off_t) state->block*TFTP_BLOCKSIZE);
+  return CURLE_OK;
 }
 
 /**********************************************************
@@ -400,17 +418,18 @@ static void tftp_rx(tftp_state_data_t *state, tftp_event_t event)
  * Event handler for the TX state
  *
  **********************************************************/
-static void tftp_tx(tftp_state_data_t *state, tftp_event_t event)
+static CURLcode tftp_tx(tftp_state_data_t *state, tftp_event_t event)
 {
   struct SessionHandle *data = state->conn->data;
   int sbytes;
   int rblock;
+  CURLcode res = CURLE_OK;
 
   switch(event) {
 
   case TFTP_EVENT_ACK:
     /* Ack the packet */
-    rblock = ntohs(state->rpacket.u.data.block);
+    rblock = getrpacketblock(&state->rpacket);
 
     if(rblock != state->block) {
       /* This isn't the expected block.  Log it and up the retry counter */
@@ -419,30 +438,44 @@ static void tftp_tx(tftp_state_data_t *state, tftp_event_t event)
       state->retries++;
       /* Bail out if over the maximum */
       if(state->retries>state->retry_max) {
-        failf(data, "%s\n",
-              "tftp_tx: giving up waiting for block %d ack",
+        failf(data, "tftp_tx: giving up waiting for block %d ack",
               state->block);
+        res = CURLE_SEND_ERROR;
       }
-      return;
+      else {
+        /* Re-send the data packet */
+        sbytes = sendto(state->sockfd, (void *)&state->spacket,
+                        4+state->sbytes, SEND_4TH_ARG,
+                        (struct sockaddr *)&state->remote_addr,
+                        state->remote_addrlen);
+        /* Check all sbytes were sent */
+        if(sbytes<0) {
+          failf(data, "%s\n", Curl_strerror(state->conn, Curl_sockerrno()));
+          res = CURLE_SEND_ERROR;
+        }
+      }
+      return res;
     }
     /* This is the expected packet.  Reset the counters and send the next
        block */
     state->block++;
     state->retries = 0;
-    state->spacket.event = htons(TFTP_EVENT_DATA);
-    state->spacket.u.ack.block = htons(state->block);
-    if(state->block > 1 && state->sbytes < 512) {
+    setpacketevent(&state->spacket, TFTP_EVENT_DATA);
+    setpacketblock(&state->spacket, state->block);
+    if(state->block > 1 && state->sbytes < TFTP_BLOCKSIZE) {
       state->state = TFTP_STATE_FIN;
-      return;
+      return CURLE_OK;
     }
-    Curl_fillreadbuffer(state->conn, 512, &state->sbytes);
-    sbytes = sendto(state->sockfd, (void *)&state->spacket,
+    res = Curl_fillreadbuffer(state->conn, TFTP_BLOCKSIZE, &state->sbytes);
+    if(res)
+      return res;
+    sbytes = sendto(state->sockfd, (void *)state->spacket.data,
                     4+state->sbytes, SEND_4TH_ARG,
                     (struct sockaddr *)&state->remote_addr,
                     state->remote_addrlen);
     /* Check all sbytes were sent */
     if(sbytes<0) {
-      failf(data, "%s\n", strerror(errno));
+      failf(data, "%s\n", Curl_strerror(state->conn, Curl_sockerrno()));
     }
     break;
 
@@ -463,7 +496,7 @@ static void tftp_tx(tftp_state_data_t *state, tftp_event_t event)
                       state->remote_addrlen);
       /* Check all sbytes were sent */
       if(sbytes<0) {
-        failf(data, "%s\n", strerror(errno));
+        failf(data, "%s\n", Curl_strerror(state->conn, Curl_sockerrno()));
       }
     }
     break;
@@ -478,7 +511,9 @@ static void tftp_tx(tftp_state_data_t *state, tftp_event_t event)
   }
 
   /* Update the progress meter */
-  Curl_pgrsSetUploadCounter(data, (curl_off_t) state->block*512);
+  Curl_pgrsSetUploadCounter(data, (curl_off_t) state->block*TFTP_BLOCKSIZE);
+
+  return res;
 }
 
 /**********************************************************
@@ -491,19 +526,20 @@ static void tftp_tx(tftp_state_data_t *state, tftp_event_t event)
 static CURLcode tftp_state_machine(tftp_state_data_t *state,
                                    tftp_event_t event)
 {
+  CURLcode res = CURLE_OK;
   struct SessionHandle *data = state->conn->data;
   switch(state->state) {
   case TFTP_STATE_START:
     DEBUGF(infof(data, "TFTP_STATE_START\n"));
-    tftp_send_first(state, event);
+    res = tftp_send_first(state, event);
     break;
   case TFTP_STATE_RX:
     DEBUGF(infof(data, "TFTP_STATE_RX\n"));
-    tftp_rx(state, event);
+    res = tftp_rx(state, event);
     break;
   case TFTP_STATE_TX:
     DEBUGF(infof(data, "TFTP_STATE_TX\n"));
-    tftp_tx(state, event);
+    res = tftp_tx(state, event);
     break;
   case TFTP_STATE_FIN:
     infof(data, "%s\n", "TFTP finished");
@@ -511,9 +547,10 @@ static CURLcode tftp_state_machine(tftp_state_data_t *state,
   default:
     DEBUGF(infof(data, "STATE: %d\n", state->state));
     failf(data, "%s\n", "Internal state machine error");
+    res = CURLE_TFTP_ILLEGAL;
     break;
   }
-  return CURLE_OK;
+  return res;
 }
 
 
@@ -530,28 +567,9 @@ CURLcode Curl_tftp_connect(struct connectdata *conn, bool *done)
   tftp_state_data_t     *state;
   int rc;
 
-  /*
-   * The TFTP code is not portable because it sends C structs directly over
-   * the wire.  Since C gives compiler writers a wide latitude in padding and
-   * aligning structs, this fails on many architectures (e.g. ARM).
-   *
-   * The only portable way to fix this is to copy each struct item into a
-   * flat buffer and send the flat buffer instead of the struct.  The
-   * alternative, trying to get the compiler to eliminate padding bytes
-   * within the struct, is a nightmare to maintain (each compiler does it
-   * differently), and is still not guaranteed to work because some
-   * architectures can't handle the resulting alignment.
-   *
-   * This check can be removed once the code has been fixed.
-   */
-  if(sizeof(struct tftp_packet) != 516) {
-    failf(conn->data, "tftp not supported on this architecture");
-    return CURLE_FAILED_INIT;
-  }
-
-  if((state = conn->proto.tftp = calloc(sizeof(tftp_state_data_t), 1))==NULL) {
+  state = conn->proto.tftp = calloc(sizeof(tftp_state_data_t), 1);
+  if(!state)
     return CURLE_OUT_OF_MEMORY;
-  }
 
   state->conn = conn;
   state->sockfd = state->conn->sock[FIRSTSOCKET];
@@ -559,17 +577,31 @@ CURLcode Curl_tftp_connect(struct connectdata *conn, bool *done)
 
 #ifdef WIN32
   /* AF_UNSPEC == 0 (from above calloc) doesn't work on Winsock */
-  ((struct sockaddr_in*)&state->local_addr)->sin_family = conn->ip_addr->ai_family;
+
+  /* NOTE: this blatantly assumes IPv4. This should be fixed! */
+  ((struct sockaddr_in*)&state->local_addr)->sin_family =
+    conn->ip_addr->ai_family;
 #endif
 
   tftp_set_timeouts(state);
 
-  /* Bind to any interface, random UDP port */
+  /* Bind to any interface, random UDP port.
+   *
+   * We once used the size of the local_addr struct as the third argument for
+   * bind() to better work with IPv6 or whatever size the struct could have,
+   * but we learned that at least Tru64, AIX and IRIX *requires* the size of
+   * that argument to match the exact size of a 'sockaddr_in' struct when
+   * running IPv4-only.
+   *
+   * Therefore we use the size from the address we connected to, which we
+   * assume uses the same IP version and thus hopefully this works for both
+   * IPv4 and IPv6...
+   */
   rc = bind(state->sockfd, (struct sockaddr *)&state->local_addr,
-            sizeof(state->local_addr));
+            conn->ip_addr->ai_addrlen);
   if(rc) {
     failf(conn->data, "bind() failed; %s\n",
-          Curl_strerror(conn,Curl_ourerrno()));
+          Curl_strerror(conn, Curl_sockerrno()));
     return CURLE_COULDNT_CONNECT;
   }
 
@@ -632,7 +664,7 @@ CURLcode Curl_tftp(struct connectdata *conn, bool *done)
 
     if(rc == -1) {
       /* bail out */
-      int error = Curl_ourerrno();
+      int error = Curl_sockerrno();
       failf(data, "%s\n", Curl_strerror(conn, error));
       event = TFTP_EVENT_ERROR;
     }
@@ -656,30 +688,46 @@ CURLcode Curl_tftp(struct connectdata *conn, bool *done)
         state->remote_addrlen = fromlen;
       }
 
-      /* The event is given by the TFTP packet time */
-      event = (tftp_event_t)ntohs(state->rpacket.event);
-
-      switch(event) {
-      case TFTP_EVENT_DATA:
-        Curl_client_write(data, CLIENTWRITE_BODY,
-                          (char *)state->rpacket.u.data.data, state->rbytes-4);
-        break;
-      case TFTP_EVENT_ERROR:
-        state->error = (tftp_error_t)ntohs(state->rpacket.u.error.code);
-        infof(conn->data, "%s\n", (char *)state->rpacket.u.error.data);
-        break;
-      case TFTP_EVENT_ACK:
-        break;
-      case TFTP_EVENT_RRQ:
-      case TFTP_EVENT_WRQ:
-      default:
-        failf(conn->data, "%s\n", "Internal error: Unexpected packet");
-        break;
+      /* Sanity check packet length */
+      if (state->rbytes < 4) {
+        failf(conn->data, "Received too short packet\n");
+        /* Not a timeout, but how best to handle it? */
+        event = TFTP_EVENT_TIMEOUT;
       }
+      else {
 
-      /* Update the progress meter */
-      Curl_pgrsUpdate(conn);
+        /* The event is given by the TFTP packet time */
+        event = (tftp_event_t)getrpacketevent(&state->rpacket);
 
+        switch(event) {
+        case TFTP_EVENT_DATA:
+          /* Don't pass to the client empty or retransmitted packets */
+          if (state->rbytes > 4 &&
+              ((state->block+1) == getrpacketblock(&state->rpacket))) {
+            code = Curl_client_write(data, CLIENTWRITE_BODY,
+                                     (char *)&state->rpacket.data[4],
+                                     state->rbytes-4);
+            if(code)
+              return code;
+          }
+          break;
+        case TFTP_EVENT_ERROR:
+          state->error = (tftp_error_t)getrpacketblock(&state->rpacket);
+          infof(conn->data, "%s\n", (char *)&state->rpacket.data[4]);
+          break;
+        case TFTP_EVENT_ACK:
+          break;
+        case TFTP_EVENT_RRQ:
+        case TFTP_EVENT_WRQ:
+        default:
+          failf(conn->data, "%s\n", "Internal error: Unexpected packet");
+          break;
+        }
+
+        /* Update the progress meter */
+        if(Curl_pgrsUpdate(conn))
+          return CURLE_ABORTED_BY_CALLBACK;
+      }
     }
 
     /* Check for transfer timeout every 10 blocks, or after timeout */
@@ -697,7 +745,9 @@ CURLcode Curl_tftp(struct connectdata *conn, bool *done)
   }
 
   /* Tell curl we're done */
-  Curl_Transfer(conn, -1, -1, FALSE, NULL, -1, NULL);
+  code = Curl_Transfer(conn, -1, -1, FALSE, NULL, -1, NULL);
+  if(code)
+    return code;
 
   /* If we have encountered an error */
   if(state->error) {
