@@ -1,5 +1,5 @@
-#ifndef __CLIENT_SETUP_H
-#define __CLIENT_SETUP_H
+#ifndef __SRC_CURL_SETUP_H
+#define __SRC_CURL_SETUP_H
 /***************************************************************************
  *                                  _   _ ____  _
  *  Project                     ___| | | |  _ \| |
@@ -20,7 +20,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: setup.h,v 1.37 2005/11/24 10:22:47 bagder Exp $
+ * $Id: setup.h,v 1.41 2006-06-08 06:12:31 bagder Exp $
  ***************************************************************************/
 
 #define CURL_NO_OLDIES
@@ -29,30 +29,47 @@
 /* Borland fix */
 #define WIN32
 #endif
+
 #if !defined(WIN32) && defined(_WIN32)
-/* This works for VS2005 on x64 */
+/* VS2005 on x64 fix */
 #define WIN32
 #endif
 
+/*
+ * Include configuration script results or hand-crafted
+ * configuration file for platforms which lack config tool.
+ */
+
 #ifdef HAVE_CONFIG_H
-#include "config.h" /* the configure script results */
+#include "config.h"
 #else
+
 #ifdef WIN32
-/* include the hand-modified win32 adjusted config.h! */
 #include "config-win32.h"
 #endif
+
 #ifdef macintosh
-/* this is not the same as Mac OS X */
 #include "config-mac.h"
 #endif
+
 #ifdef __riscos__
 #include "config-riscos.h"
 #endif
+
 #ifdef __amigaos__
 #include "config-amigaos.h"
 #endif
+
+#ifdef TPF
+#include "config-tpf.h"
+/* change which select is used for the curl command line tool */
+#define select(a,b,c,d,e) tpf_select_bsd(a,b,c,d,e)
+/* and turn off the progress meter */
+#define CONF_DEFAULT (0|CONF_NOPROGRESS)
 #endif
 
+#endif /* HAVE_CONFIG_H */
+
 #if defined(CURLDEBUG) && defined(CURLTOOLDEBUG)
 /* This is an ugly hack for CURLDEBUG conditions only. We need to include
    the file here, since it might set the _FILE_OFFSET_BITS define, which must
@@ -60,6 +77,32 @@
 #include "../lib/setup.h"
 #endif
 
+/* 
+ * Include header files for windows builds before redefining anything.
+ * Use this preproessor block only to include or exclude windows.h, 
+ * winsock2.h, ws2tcpip.h or winsock.h. Any other windows thing belongs 
+ * to any other further and independant block.  Under Cygwin things work
+ * just as under linux (e.g. <sys/socket.h>) and the winsock headers should
+ * never be included.
+ */
+
+#if defined(HAVE_WINDOWS_H) && !defined(__CYGWIN__)
+#  ifndef WIN32_LEAN_AND_MEAN
+#    define WIN32_LEAN_AND_MEAN
+#  endif
+#  include <windows.h>
+#  ifdef HAVE_WINSOCK2_H
+#    include <winsock2.h>
+#    ifdef HAVE_WS2TCPIP_H
+#       include <ws2tcpip.h>
+#    endif
+#  else
+#    ifdef HAVE_WINSOCK_H
+#      include <winsock.h>
+#    endif
+#  endif
+#endif
+
 #include <stdio.h>
 
 #ifdef __TANDEM
@@ -74,7 +117,7 @@
 #endif
 
 #if !defined(__cplusplus) && !defined(__BEOS__) && !defined(typedef_bool)
-typedef char bool;
+typedef unsigned char bool;
 #define typedef_bool
 #endif
 
@@ -129,4 +172,9 @@ int fileno( FILE *stream);
 #define SIZEOF_CURL_OFF_T sizeof(curl_off_t)
 #endif
 
-#endif /* __SETUP_H */
+#ifndef UNPRINTABLE_CHAR
+/* define what to use for unprintable characters */
+#define UNPRINTABLE_CHAR '.'
+#endif
+
+#endif /* __SRC_CURL_SETUP_H */
